# javascript-problems-and-solutions

[![npm version](https://badge.fury.io/js/javascript-problems-and-solutions.svg)](https://badge.fury.io/js/javascript-problems-and-solutions)
[![Build Status](https://travis-ci.org/jeantimex/javascript-problems-and-solutions.svg?branch=master)](https://travis-ci.org/jeantimex/javascript-problems-and-solutions)
[![Coverage Status](https://coveralls.io/repos/github/jeantimex/javascript-problems-and-solutions/badge.svg?branch=code-coverage)](https://coveralls.io/github/jeantimex/javascript-problems-and-solutions?branch=code-coverage)
[![devDependencies Status](https://david-dm.org/jeantimex/javascript-problems-and-solutions/dev-status.svg)](https://david-dm.org/jeantimex/javascript-problems-and-solutions?type=dev)
[![code style: prettier](https://img.shields.io/badge/code_style-prettier-ff69b4.svg)](https://github.com/prettier/prettier)

A collection of JavaScript problems and solutions for studying algorithms.

<img alt="cover" src="https://user-images.githubusercontent.com/565300/33303581-c55a1280-d3b8-11e7-8e78-6879731416f1.png" width="600" />

## Data Structures

- [Stack](src/common/stack.js)
- [Queue](src/common/queue.js)
- [Trie](src/common/trie.js)
- [Priority Queue](src/common/priority-queue.js)
- [Tree Node](src/common/tree-node.js)
- [Linked List Node](src/common/list-node.js)

## Problems

### Array

- [Median of Two Sorted Arrays](src/array/median-of-two-sorted-arrays.js)
- [Plus One](src/array/plus-one.js)
- [Trapping Rain Water](src/array/trapping-rain-water)
- [Merge Intervals](src/array/merge-intervals.js)
- [Spiral Matrix](src/array/spiral-matrix.js)
- [Summary Ranges](src/array/summary-ranges.js)
- [Find All Numbers Disappeared in an Array](src/array/find-all-numbers-disappeared-in-an-array.js)
- [Game of Life](src/array/game-of-life.js)
- [Next Permutation](src/array/next-permutation.js)
- [Find Peak Element](src/array/find-peak-element.js)
- [Wiggle Sort](src/array/wiggle-sort.js)
- [Valid Triangle Number](src/array/valid-triangle-number.js)
- [Find Anagram Mappings](src/array/find-anagram-mappings.js)
- [K Empty Slots](src/array/k-empty-slots.js)

### Matrix

- [Island Perimeter](src/matrix/island-perimeter)

### String

- [Regular Expression Matching](src/backtracking/regular-expression-matching.js)
- [Letter Combinations of a Phone Number](src/backtracking/letter-combinations-of-a-phone-number.js)
- [Valid Parentheses](src/string/valid-parentheses.js)
- [Generate Parentheses](src/backtracking/generate-parentheses.js)
- [Generate Parentheses II](src/backtracking/generate-parentheses-ii.js)
- [Shortest Palindrome](src/string/shortest-palindrome.js)
- [Palindrome Pairs](src/trie/palindrome-pairs.js)
- [First Unique Character in a String](src/string/first-unique-character-in-a-string.js)
- [Judge Route Circle](src/string/judge-route-circle.js)
- [Reverse Vowels of a String](src/string/reverse-vowels-of-a-string.js)
- [Add Bold Tag in String](src/string/add-bold-tag-in-string.js)
- [Minimum Window Substring](src/string/minimum-window-substring.js)
- [Longest Absolute File Path](src/string/longest-absolute-file-path.js)

### Sorting

- [Insertion Sort](src/sorting/insertion-sort.js)
- [Merge Sort](src/sorting/merge-sort.js)
- [Quick Sort](src/sorting/quick-sort.js)
- [Quick Sort II](src/sorting/quick-sort-ii.js)
- [Heap Sort](src/sorting/heap-sort.js)

### Binary Search

- [Binary Search](src/binary-search/binary-search.js)
- [Search in Rotated Sorted Array](src/binary-search/search-in-rotated-sorted-array.js)
- [Search in Rotated Sorted Array II](src/binary-search/search-in-rotated-sorted-array-ii.js)
- [Divide Two Integers](src/binary-search/divide-two-integers.js)
- [Power(x, n)](src/binary-search/powerx-n.js)

### Tree

- [Add One Row to Tree](src/tree/add-one-row-to-tree.js)
- [Balanced Binary Tree](src/tree/balanced-binary-tree.js)
- [Binary Search Tree I - Search](src/tree/binary-search-tree-i-search.js)
- [Binary Search Tree II - Insert](src/tree/binary-search-tree-ii-insert.js)
- [Binary Search Tree II - Delete](src/tree/binary-search-tree-iii-delete.js)
- [Binary Search Tree Iterator](src/tree/binary-search-tree-iterator.js)
- [Binary Tree Inorder Traversal](src/tree/binary-tree-inorder-traversal.js)
- [Binary Tree Level Order Traversal](src/tree/binary-tree-level-order-traversal.js)
- [Binary Tree Longest Consecutive Sequence](src/tree/binary-tree-longest-consecutive-sequence.js)
- [Binary Tree Longest Consecutive Sequence II](src/tree/binary-tree-longest-consecutive-sequence-ii.js)
- [Binary Tree Maximum Path Sum](src/tree/binary-tree-maximum-path-sum.js)
- [Binary Tree Paths](src/tree/binary-tree-paths.js)
- [Binary Tree Postorder Traversal](src/tree/binary-tree-postorder-traversal.js)
- [Binary Tree Preorder Traversal](src/tree/binary-tree-preorder-traversal.js)
- [Binary Tree Upside Down](src/tree/binary-tree-upside-down.js)
- [Binary Tree Zigzag Level Order Traversal](src/tree/binary-tree-zigzag-level-order-traversal.js)
- [Closest Binary Search Tree Value](src/tree/closest-binary-search-tree-value.js)
- [Closest Binary Search Tree Value II](src/tree/closest-binary-search-tree-value-ii.js)
- [Construct Binary Tree From Preorder and Inorder Traversal](src/tree/construct-binary-tree-from-preorder-and-inorder-traversal.js)
- [Construct Binary Tree From Postorder and Inorder Traversal](src/tree/construct-binary-tree-from-inorder-and-postorder-traversal.js)
- [Convert BST to Greater Tree](src/tree/convert-bst-to-greater-tree.js)
- [Convert Sorted Array to Binary Search Tree](src/tree/convert-sorted-array-to-binary-search-tree.js)
- [Count Univalue Subtrees](src/tree/count-univalue-subtrees.js)
- [Delete Node in a BST](src/tree/delete-node-in-a-bst.js)
- [Diameter of Binary Tree](src/tree/diameter-of-binary-tree.js)
- [Find Bottom Left Tree Value](src/tree/find-bottom-left-tree-value.js)
- [Find Largest Value in Each Tree Row](src/tree/find-largest-value-in-each-tree-row.js)
- [Find Mode in Binary Search Tree](src/tree/find-mode-in-binary-search-tree.js)
- [Flatten Binary Tree to Linked List](src/tree/flatten-binary-tree-to-linked-list.js)
- [House Robber III](src/tree/house-robber-iii.js)
- [Inorder Predecessor in BST](src/tree/inorder-predecessor-in-bst.js)
- [Inorder Successor in BST](src/tree/inorder-successor-in-bst.js)
- [K-th ancestor of a node in Binary Tree](src/tree/k-th-ancestor-of-a-node-in-binary-tree.js)
- [Kill Process](src/tree/kill-process.js)
- [Longest Univalue Path](src/tree/longest-univalue-path.js)
- [Lowest Common Ancestor of A Binary Search Tree](src/tree/lowest-common-ancestor-of-a-binary-search-tree.js)
- [Lowest Common Ancestor of a Binary Tree](src/tree/lowest-common-ancestor-of-a-binary-tree.js)
- [Maximum Binary Tree](src/tree/maximum-binary-tree.js)
- [Maximum Depth of Binary Tree](src/tree/maximum-depth-of-binary-tree.js)
- [Maximum Width of Binary Tree](src/tree/maximum-width-of-binary-tree.js)
- [Minimum Depth of Binary Tree](src/tree/minimum-depth-of-binary-tree.js)
- [Most Frequent Subtree Sum](src/tree/most-frequent-subtree-sum.js)
- [Path Sum](src/tree/path-sum.js)
- [Path Sum II](src/tree/path-sum-ii.js)
- [Path Sum III](src/tree/path-sum-iii.js)
- [Path Sum IV](src/tree/path-sum-iv.js)
- [Populating Next Right Pointers in Each Node](src/tree/populating-next-right-pointers-in-each-node.js)
- [Populating Next Right Pointers in Each Node II](src/tree/populating-next-right-pointers-in-each-node-ii.js)
- [Recover Binary Search Tree](src/tree/recover-binary-search-tree.js)
- [Same Tree](src/tree/same-tree.js)
- [Serialize and Deserialize BST](src/tree/serialize-and-deserialize-bst.js)
- [Serialize and Deserialize Binary Tree](src/tree/serialize-and-deserialize-binary-tree.js)
- [Subtree of Another Tree](src/tree/subtree-of-another-tree.js)
- [Sum Root to Leaf Numbers](src/tree/sum-root-to-leaf-numbers.js)
- [Symmetric Tree](src/tree/symmetric-tree.js)
- [Trim a Binary Search Tree](src/tree/trim-a-binary-search-tree.js)
- [Unique Binary Search Trees](src/tree/unique-binary-search-trees.js)
- [Unique Binary Search Trees II](src/tree/unique-binary-search-trees-ii.js)
- [Univalue Tree](src/tree/univalue-tree.js)
- [Validate Binary Search Tree](src/tree/validate-binary-search-tree.js)
- [Verify Preorder Sequence in Binary Search Tree](src/tree/verify-preorder-sequence-in-binary-search-tree.js)

### Graph

- [Clone Graph](src/graph/clone-graph.js)
- [Course Schedule](src/graph/course-schedule.js)
- [Course Schedule II](src/graph/course-schedule-ii.js)
- [Alien Dictionary](src/graph/alien-dictionary.js)
- [Alien Dictionary II](src/graph/alien-dictionary-ii.js)
- [Graph Valid Tree](src/graph/graph-valid-tree.js)
- [Number of Connected Components in an Undirected Graph](src/graph/number-of-connected-components-in-an-undirected-graph.js)
- [Reconstruct Itinerary](src/graph/reconstruct-itinerary.js)
- [Evaluate Division](src/graph/evaluate-division.js)
- [Sequence Reconstruction](src/graph/sequence-reconstruction.js)
- [Minimum Height Trees](src/graph/minimum-height-trees.js)
- [Redundant Connection](src/graph/redundant-connection.js)
- [Redundant Connection II](src/graph/redundant-connection-ii.js)

### Trie (Prefix Tree)

- [Implement Trie (Prefix Tree)](src/common/trie.js)
- [Add and Search Word](src/trie/add-and-search-word-data-structure-design.js)
- [Word Search II](src/trie/word-search-ii.js)
- [Longest Word in Dictionary](src/trie/longest-word-in-dictionary.js)
- [Map Sum Pairs](src/trie/map-sum-pairs.js)
- [Implement Magic Dictionary](src/trie/implement-magic-dictionary.js)
- [Replace Words](src/trie/replace-words.js)
- [Top K Frequent Words](src/trie/top-k-frequent-words.js)
- [Concatenated Words](src/trie/concatenated-words.js)
- [Palindrome Pairs](src/trie/palindrome-pairs.js)
- [Design Search Autocomplete System](src/trie/design-search-autocomplete-system.js)
- [Word Squares](src/trie/word-squares.js)
- [Maximum XOR of Two Numbers in an Array](src/trie/maximum-xor-of-two-numbers-in-an-array.js)
- [Add Bold Tag in String](src/trie/add-bold-tag-in-string.js)

### Backtracking

- [Subsets](src/backtracking/subsets.js)
- [Subsets II](src/backtracking/subsets-ii.js)
- [Combinations](src/backtracking/combinations.js)
- [Combination Sum](src/backtracking/combination-sum.js)
- [Combination Sum II](src/backtracking/combination-sum-ii.js)
- [Combination Sum III](src/backtracking/combination-sum-iii.js)
- [Permutations](src/backtracking/permutations.js)
- [Permutations II](src/backtracking/permutations-ii.js)
- [N-Queens](src/backtracking/n-queens.js)
- [N-Queens II](src/backtracking/n-queens-ii.js)
- [Generate Parentheses](src/backtracking/generate-parentheses.js)
- [Generate Parentheses II](src/backtracking/generate-parentheses-ii.js)
- [Word Search](src/backtracking/word-search.js)
- [Word Search II](src/trie/word-search-ii.js)
- [Sudoku Solver](src/backtracking/sudoku-solver.js)
- [Regular Expression Matching](src/backtracking/regular-expression-matching.js)
- [Wildcard Matching](src/backtracking/wildcard-matching.js)
- [Letter Combinations of a Phone Number](src/backtracking/letter-combinations-of-a-phone-number.js)
- [Letter Case Permutation](src/backtracking/letter-case-permutation.js)
- [Count Numbers with Unique Digits](src/backtracking/count-numbers-with-unique-digits.js)
- [Restore IP Addresses](src/backtracking/restore-ip-addresses.js)

### Breadth First Search

- [Shortest Path in Maze](src/bfs/shortest-path-in-maze.js)
- [Shortest Path in Maze II](src/bfs/shortest-path-in-maze-ii.js)
- [Number of Islands](src/bfs/number-of-islands.js)

### Depth First Search

- [Number of Islands](src/dfs/number-of-islands.js)
- [Next Closest Time](src/dfs/next-closest-time.js)

### Linked List

- [Add Two Numbers](src/linked-list/add-two-numbers.js)
- [Merge K Sorted Lists](src/linked-list/merge-k-sorted-lists.js)

### Dynamic Programming

- [Longest Increasing Subsequence](src/dynamic-programming/longest-increasing-subsequence.js)
- [Longest Palindromic Subsequence](src/dynamic-programming/longest-palindromic-subsequence.js)
- [Longest Palindromic Substring](src/dynamic-programming/longest-palindromic-substring.js)
- [Longest Consecutive Sequence](src/dynamic-programming/longest-consecutive-sequence.js)
- [Regular Expression Matching](src/backtracking/regular-expression-matching.js)
- [Wildcard Matching](src/backtracking/wildcard-matching.js)
- [Word Break](src/dynamic-programming/word-break.js)
- [Word Break II](src/dynamic-programming/word-break-ii.js)
- [Best Time to Buy and Sell Stock](src/dynamic-programming/best-time-to-buy-and-sell-stock.js)
- [Best Time to Buy and Sell Stock II](src/dynamic-programming/best-time-to-buy-and-sell-stock-ii.js)
- [Best Time to Buy and Sell Stock III](src/dynamic-programming/best-time-to-buy-and-sell-stock-iii.js)
- [Best Time to Buy and Sell Stock IV](src/dynamic-programming/best-time-to-buy-and-sell-stock-iv.js)
- [Best Time to Buy and Sell Stock with Cooldown](src/dynamic-programming/best-time-to-buy-and-sell-stock-with-cooldown.js)
- [Best Time to Buy and Sell Stock with Transaction Fee](src/dynamic-programming/best-time-to-buy-and-sell-stock-with-transaction-fee.js)
- [Gas Station III](src/dynamic-programming/gas-station-iii.js)
- [Perfect Squares](src/dynamic-programming/perfect-squares.js)
- [Subset Sum Problem](src/dynamic-programming/subset-sum-problem.js)
- [Matrix Chain Multiplication](src/dynamic-programming/matrix-chain-multiplication.js)
- [Burst Balloons](src/dynamic-programming/burst-balloons.js)

### Greedy

- [Gas Station](src/greedy/gas-station.js)
- [Gas Station II](src/greedy/gas-station-ii.js)
- [Queue Reconstruction by Height](src/greedy/queue-reconstruction-by-height.js)
- [K Workers](src/greedy/k-workers.js)

### System Design

- [Encode and Decode TinyURL](src/system-design/encode-and-decode-tinyurl.js)
- [LRU Cache](src/system-design/lru-cache.js)
- [LFU Cache](src/system-design/lfu-cache.js)
- [Min Stack](src/system-design/min-stack.js)
- [Max Stack](src/system-design/max-stack.js)

## Author

I solve algorithm problems using JavaSript and Java, you can find all my LeetCode solutions and posts [here](https://discuss.leetcode.com/user/jeantimex).

## Support

<a href="https://paypal.me/jeantimex/3">
  <img alt="But Me a Coffee" src="https://az743702.vo.msecnd.net/cdn/kofi4.png?v=0" width="200" />
</a>

## LICENSE

MIT License

Copyright (c) 2017 Yong Su

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
